@@ -1,4 +1,4 @@
-# Copyright 2020 Cortex Labs, Inc.
+# Copyright 2021 Cortex Labs, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -20,7 +20,7 @@ from cortex.binary import run_cli
 from cortex.exceptions import NotFound
 
 
-def client(env: str):
+def client(env: str) -> Client:
     """
     Initialize a client based on the specified environment.
 
@@ -85,7 +85,7 @@ def cluster_client(
         ]
     run_cli(cli_args, hide_output=True)
 
-    return Client(name)
+    return client(name)
 
 
 def env_list() -> list:
