@@ -1,4 +1,4 @@
-# Copyright 2020 Cortex Labs, Inc.
+# Copyright 2021 Cortex Labs, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -33,6 +33,13 @@ from cortex import util
 EXPECTED_PYTHON_VERSION = "3.6.9"
 
 
+def cli_config_dir() -> Path:
+    cli_config_dir = os.environ.get("CORTEX_CLI_CONFIG_DIR", "")
+    if cli_config_dir == "":
+        return Path.home() / ".cortex"
+    return Path(cli_config_dir).expanduser().resolve()
+
+
 class Client:
     def __init__(self, env: dict):
         """
@@ -49,6 +56,7 @@ class Client:
         self,
         api_spec: dict,
         predictor=None,
+        task=None,
         requirements=[],
         conda_packages=[],
         project_dir: Optional[str] = None,
@@ -59,8 +67,9 @@ class Client:
         Deploy an API.
 
         Args:
-            api_spec: A dictionary defining a single Cortex API. See https://docs.cortex.dev/v/0.26/ for schema.
-            predictor: A Cortex Predictor class implementation. Not required when deploying a traffic splitter.
+            api_spec: A dictionary defining a single Cortex API. See https://docs.cortex.dev/v/0.27/ for schema.
+            predictor: A Cortex Predictor class implementation. Not required for TaskAPI/TrafficSplitter kinds.
+            task: A callable class/function implementation. Not required for RealtimeAPI/BatchAPI/TrafficSplitter kinds.
             requirements: A list of PyPI dependencies that will be installed before the predictor class implementation is invoked.
             conda_packages: A list of Conda dependencies that will be installed before the predictor class implementation is invoked.
             project_dir: Path to a python project.
@@ -76,10 +85,15 @@ class Client:
                 "`wait` flag is not supported for clusters on GCP, please set the `wait` flag to false"
             )
 
-        if project_dir is not None and predictor is not None:
-            raise ValueError(
-                "`predictor` and `project_dir` parameters cannot be specified at the same time, please choose one"
-            )
+        if project_dir is not None:
+            if predictor is not None:
+                raise ValueError(
+                    "`predictor` and `project_dir` parameters cannot be specified at the same time, please choose one"
+                )
+            if task is not None:
+                raise ValueError(
+                    "`task` and `project_dir` parameters cannot be specified at the same time, please choose one"
+                )
 
         if project_dir is not None:
             cortex_yaml_path = os.path.join(project_dir, f".cortex-{uuid.uuid4()}.yaml")
@@ -88,10 +102,31 @@ class Client:
                 yaml.dump([api_spec], f)  # write a list
                 return self._deploy(cortex_yaml_path, force, wait)
 
+        api_kind = api_spec.get("kind")
+        if api_kind == "TrafficSplitter":
+            if predictor:
+                raise ValueError(f"`predictor` parameter cannot be specified for {api_kind} kind")
+            if task:
+                raise ValueError(f"`task` parameter cannot be specified for {api_kind} kind")
+        elif api_kind == "TaskAPI":
+            if predictor:
+                raise ValueError(f"`predictor` parameter cannnot be specified for {api_kind} kind")
+            if task is None:
+                raise ValueError(f"`task` parameter must be specified for {api_kind} kind")
+        elif api_kind in ["BatchAPI", "RealtimeAPI"]:
+            if not predictor:
+                raise ValueError(f"`predictor` parameter must be specified for {api_kind}")
+            if task:
+                raise ValueError(f"`task` parameter cannot be specified for {api_kind}")
+        else:
+            raise ValueError(
+                f"invalid {api_kind} kind, `api_spec` must have the `kind` field set to one of the following kinds: {['TrafficSplitter', 'TaskAPI', 'BatchAPI', 'RealtimeAPI']}"
+            )
+
         if api_spec.get("name") is None:
             raise ValueError("`api_spec` must have the `name` key set")
 
-        project_dir = Path.home() / ".cortex" / "deployments" / api_spec["name"]
+        project_dir = cli_config_dir() / "deployments" / api_spec["name"]
 
         if project_dir.exists():
             shutil.rmtree(str(project_dir))
@@ -100,7 +135,7 @@ class Client:
 
         cortex_yaml_path = os.path.join(project_dir, "cortex.yaml")
 
-        if predictor is None:
+        if api_kind == "TrafficSplitter":
             # for deploying a traffic splitter
             with open(cortex_yaml_path, "w") as f:
                 yaml.dump([api_spec], f)  # write a list
@@ -127,23 +162,35 @@ class Client:
             with open(project_dir / "conda-packages.txt", "w") as conda_file:
                 conda_file.write("\n".join(conda_packages))
 
-        if not inspect.isclass(predictor):
-            raise ValueError("predictor parameter must be a class definition")
+        if api_kind in ["BatchAPI", "RealtimeAPI"]:
+            if not inspect.isclass(predictor):
+                raise ValueError("`predictor` parameter must be a class definition")
 
-        with open(project_dir / "predictor.pickle", "wb") as pickle_file:
-            dill.dump(predictor, pickle_file)
-            if api_spec.get("predictor") is None:
-                api_spec["predictor"] = {}
+            with open(project_dir / "predictor.pickle", "wb") as pickle_file:
+                dill.dump(predictor, pickle_file)
+                if api_spec.get("predictor") is None:
+                    api_spec["predictor"] = {}
 
-            if predictor.__name__ == "PythonPredictor":
-                predictor_type = "python"
-            if predictor.__name__ == "TensorFlowPredictor":
-                predictor_type = "tensorflow"
-            if predictor.__name__ == "ONNXPredictor":
-                predictor_type = "onnx"
+                if predictor.__name__ == "PythonPredictor":
+                    predictor_type = "python"
+                if predictor.__name__ == "TensorFlowPredictor":
+                    predictor_type = "tensorflow"
+                if predictor.__name__ == "ONNXPredictor":
+                    predictor_type = "onnx"
 
-            api_spec["predictor"]["path"] = "predictor.pickle"
-            api_spec["predictor"]["type"] = predictor_type
+                api_spec["predictor"]["path"] = "predictor.pickle"
+                api_spec["predictor"]["type"] = predictor_type
+
+        if api_kind == "TaskAPI":
+            if not callable(task):
+                raise ValueError(
+                    "`task` parameter must be a callable (e.g. a function definition or a class definition called `Task` with a `__call__` method implemented"
+                )
+            with open(project_dir / "task.pickle", "wb") as pickle_file:
+                dill.dump(task, pickle_file)
+                if api_spec.get("definition") is None:
+                    api_spec["definition"] = {}
+                api_spec["definition"]["path"] = "task.pickle"
 
         with open(cortex_yaml_path, "w") as f:
             yaml.dump([api_spec], f)  # write a list
@@ -205,7 +252,7 @@ class Client:
         env = os.environ.copy()
         env["CORTEX_CLI_INVOKER"] = "python"
         process = subprocess.Popen(
-            [get_cli_path(), "logs", "--env", self.env_name, api_name],
+            [get_cli_path(), "logs", "--env", self.env_name, api_name, "-y"],
             stderr=subprocess.STDOUT,
             stdout=subprocess.PIPE,
             encoding="utf8",
@@ -260,7 +307,7 @@ class Client:
         Get information about a submitted job.
 
         Args:
-            api_name: Name of the Batch API.
+            api_name: Name of the Batch/Task API.
             job_id: Job ID.
 
         Returns:
@@ -296,9 +343,7 @@ class Client:
             force: Override an already in-progress API update.
         """
 
-        cortex_yaml_file = (
-            Path.home() / ".cortex" / "deployments" / f"cortex-{str(uuid.uuid4())}.yaml"
-        )
+        cortex_yaml_file = cli_config_dir() / "deployments" / f"cortex-{str(uuid.uuid4())}.yaml"
         with util.open_temporarily(cortex_yaml_file, "w") as f:
             yaml.dump([api_spec], f)
             args = ["patch", cortex_yaml_file, "--env", self.env_name, "-o", "json"]
@@ -337,7 +382,7 @@ class Client:
         Stop a running job.
 
         Args:
-            api_name: Name of the Batch API.
+            api_name: Name of the Batch/Task API.
             job_id: ID of the Job to stop.
         """
         args = [
@@ -362,7 +407,7 @@ class Client:
         Args:
             api_name: Name of the API.
         """
-        args = ["logs", api_name, "--env", self.env_name]
+        args = ["logs", api_name, "--env", self.env_name, "-y"]
         run_cli(args)
 
     def stream_job_logs(
@@ -377,5 +422,5 @@ class Client:
             api_name: Name of the Batch API.
             job_id: Job ID.
         """
-        args = ["logs", api_name, job_id, "--env", self.env_name]
+        args = ["logs", api_name, job_id, "--env", self.env_name, "-y"]
         run_cli(args)
